###############################################################
#                                      __  __  _              #
#            ___  ____ __  _ ___  ___ / /_ \ \/ /             #
#           / _ \' __/ _ `/ / -_)/ __/  _/  )  (              #
#          / .__/_/  \___/ /\___/\__/\__/  /_/\_\             #
#         /_/ -------- /__/ ---------------------             #
#                                                             #
#     Explainer v2.00 by Mentor <mentor@projectx.mx.dk>       #
#                 http://www.projectx.mx.dk/                  #
###############################################################

### -----------------------------------------------------------
###                       Description
### -----------------------------------------------------------
###
### This Tcl script allows you to add and delete terms and 
### their explanations/descriptions to a database. At any time
### a user can query this database to retrieve an explanation.
### (See 'Command reference' for a quick overview of all 
### available commands.) 
### This Tcl script can be used in help or support channels 
### where you can setup a database of standard replies on 
### frequently asked questions.  In non-support channels this 
### script is often used for entertainment by adding all kind
### funny and remarkable explanations to their explanation 
### database.
###
### Before we decided to release this script, we thoroughly
### tested this script on a wide range of Eggdrops, going from
### version 1.1.5 to 1.3.21.
###
### Bugreports and constructive critism for features to add
### can be send to mentor@projectx.mx.dk.  If you provide us 
### with ideas for extra features, you have a big chance we'll 
### add them for you in a next version/release of this script.
### Also, if you made a valuable explanation file, send it to
### us and we will consider to distribute it (incl. some 
### credits) along with this Tcl script.

### -----------------------------------------------------------
###                   User-defined settings
### -----------------------------------------------------------
###
### Set the filename (and the path, relative to the eggdrop's 
### root directory) of the database file that should be used to
### retrieve and store all data.  If there is no existing 
### database with the specified filename, the script will 
### create and setup a new database file for you.

set explfile "./botnick.expl"

### Set the command character that will proceed all public 
### commands.
###   Example: 
###   If you set it to "!", the command to list all available
###   explanations will be "!ls".  On the other hand, if you
###   set the command character (cc) to "@", the same command
###   would be "@ls".

set cc "!"

### Set the maximum number of terms the bot is allowed to reply
### on a public !ls query.  This feature has been added on
### request of bot admins who reported problems with large 
### database files of more then 500 entries.  
### However, if you don't have such a hugh database file you
### can disable this feature by setting 'maxterms' to 0.

set maxterms 40


### -----------------------------------------------------------
###                     Command reference
### -----------------------------------------------------------
###  PUBLIC COMMANDS:
###   !expl <term>            -  explain    -  no flag required
###   !add <term> <expl>      -  add        -  +f flag required
###   !del <term>             -  delete     -  +f flag required
###   !ls [wilcards]          -  list       -  no flag required
###
###  PARTY-LINE COMMANDS:
###   .expl <term> [channel]  -  explain    -  no flag required
###   .add <term> <expl>      -  add        -  +f flag required
###   .del <term>             -  delete     -  +f flag required
###   .ls [wilcards]          -  list       -  no flag required


###  Note 1: Both list queries, resp. !ls and .ls can be in the 
###          following formats:
###             !ls, !ls t*, !ls t*m, !ls t?rm, !ls *t?r*
###  Note 2: Only users with the +f flag (friend) set, are able
###          (allowed) to add and delete explanations.
###  Note 3: From the party-line, '.expl term #channel' will
###          display the explanation in the specified channel,
###          while '.expl term' will display the explanation
###          on the party-line.
###  Note 4: In the above command reference, the command 
###          character for the public commands is set to "!", 
###          of course this may differ depending on your 
###          personal settings above.

### -----------------------------------------------------------
###                       About ProjectX
### -----------------------------------------------------------
###
### We are an organised group of scripters determined to help
### each other create, develop and refine scripts. We bundle  
### our knowledge, so that the inexperienced scripters may 
### benefit from our experienced crew-members and our  
### experienced members benefit from the enthousiasm and fresh  
### insights from our less-experienced members. We are not 
### trying to create the 'ULTIMATE' script, there is no such
### thing. We concentrate on creating good, reliable, 
### compatible and useful scripts for Eggdrops.
### You can contact ProjectX in the following ways:
###  -  Visit our homepage..: http://www.projectx.mx.dk/
###  -  Vist us on IRC......: #projectx (GalaxyNet)
###  -  Drop us an e-mail...: info@projectx.mx.dk

###############################################################
########### Don't change anything below this line! ############
###############################################################

set ver "2.00"

bind pub - ${cc}expl pxexpl:pub:expl
bind pub f ${cc}add pxexpl:pub:add
bind pub f ${cc}del pxexpl:pub:del
bind pub - ${cc}ls pxexpl:pub:ls

bind dcc - expl pxexpl:dcc:expl
bind dcc f add pxexpl:dcc:add
bind dcc f del pxexpl:dcc:del
bind dcc - ls pxexpl:dcc:ls

proc pxexpl:pub:expl {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#$nick# expl $arg"
  if {$arg != ""} {
    set expl [pxexpl:get [lindex $arg 0]]  
    if {$expl == 0} { 
      puthelp "NOTICE $nick :\[expl\] Error: no such term.  Use '${cc}ls' to get a list of all available explanations." 
    } else {
      puthelp "PRIVMSG $chan :[lindex $arg 0]: [lrange $expl 1 end]" 
    }
  } else {
    puthelp "NOTICE $nick :\[expl\] Usage: ${cc}expl <term>"
  }
}

proc pxexpl:pub:add {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#$handle# add [lindex $arg 0]"
  if {([lindex $arg 0] != "") && ([lindex $arg 1] != "")} {
    if {[string length [lindex $arg 0]] < 23} {
      regsub -all ";" $arg ":" arg
      regsub -all \" $arg "'" arg
      if {([pxexpl:add [lindex $arg 0] [lrange $arg 1 end] $handle] == 1)} { 
        puthelp "NOTICE $nick :\[add\] Explanation has been added." 
      } else { 
        puthelp "NOTICE $nick :\[add\] Error: unable to add a duplicate term." 
      }
    } else { 
      puthelp "NOTICE $nick :\[add\] Error: term too long." 
    }
  } else { 
     puthelp "NOTICE $nick :\[add\] Usage: ${cc}add <term> <expl>" 
  }
}

proc pxexpl:pub:del {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#$handle# del $arg"
  if {$arg == ""} { 
    puthelp "NOTICE $nick :\[del\] Usage: ${cc}del <term>" 
  } elseif {[pxexpl:del [lindex $arg 0]]} { 
    puthelp "NOTICE $nick :\[del\] Explanation has been deleted." 
  } else { 
    puthelp "NOTICE $nick :\[del\] Error: no such term." 
  }
}

proc pxexpl:pub:ls {nick uhost handle chan arg} {
  global explfile maxterms cc
  putcmdlog "#$nick# ls $arg"
  set list [list]
  set total 0
  if {$arg == ""} {set arg "*"}
  set fd [open $explfile r]
  while {![eof $fd]} {
    set term [pxexpl:term [gets $fd]]
    if {$term != 0} {
      if {[string match [string tolower $arg] $term]} { 
        lappend list $term
      }
      incr total
    }
  }
  close $fd
  set matches [llength $list]
  if {$matches == 0} { 
    puthelp "NOTICE $nick :\[ls\] None of $total available terms matched your query." 
  } elseif {($matches > $maxterms) && ($maxterms > 0)} { 
    puthelp "NOTICE $nick :\[ls\] Too many terms matched your query, please refine your query by using wildcards."
  } else { 
    puthelp "NOTICE $nick :\[ls\] \037$matches of $total terms matched your query:\037"
    set line [list]
    foreach term $list {
      if {[expr [string length $line] + [string length $term]] > 60} { 
        puthelp "NOTICE $nick :\[ls\] $line $term"
        set line [list]
      } else {
        set i [expr (11 * (([string length $term] / 12) + 1)) + ([string length $term] / 12)]
        while {[string length $term] < $i} { append term " " }
        append line " $term"
      }
    }
    if {[llength $line] != 0} { puthelp "NOTICE $nick :\[ls\] $line" }
    if {[matchattr $handle f]} {
      puthelp "NOTICE $nick :\[ls\] See also: ${cc}expl <term>, ${cc}add <term> <expl>, ${cc}del <term>"
    } else {
      puthelp "NOTICE $nick :\[ls\] See also: ${cc}expl <term>"
    }
  }
}

proc pxexpl:dcc:expl {handle idx arg} {  
  global explfile
  putcmdlog "#$handle# expl $arg"
  if {$arg!=""} {
    set expl [pxexpl:get [lindex $arg 0]]  
    if {$expl == 0} { 
      putdcc $idx "\[expl\] Error: no such term." 
    } elseif {([lindex $arg 1] == "")} {
      putdcc $idx "\[expl\] [lindex $arg 0]: [lrange $expl 1 end]  (Author: [lindex $expl 0])"
    } elseif {[validchan [lindex $arg 1]]} {
      puthelp "PRIVMSG [lindex $arg 1] :[lindex $arg 0]: [lrange $expl 1 end]" 
    } else {
      putdcc $idx "\[expl\] Error: no such channel."
    }
  } else {
    putdcc $idx "\[expl\] Usage: .expl <term> \[channel\]"
  }
}

proc pxexpl:dcc:add {handle idx arg} {
  global explfile
  putcmdlog "#$handle# add [lindex $arg 0]"
  if {([lindex $arg 0] != "") && ([lindex $arg 1] != "")} {
    if {[string length [lindex $arg 0]] < 23} {
      regsub -all ";" $arg ":" arg
      regsub -all \" $arg "'" arg
      if {([pxexpl:add [lindex $arg 0] [lrange $arg 1 end] $handle] == 1)} { 
        putdcc $idx "\[add\] Explanation has been added." 
      } else { 
        putdcc $idx "\[add\] Error: unable to add a duplicate term." 
      }
    } else { 
        putdcc $idx "\[add\] Error: term too long." 
    }
  } else { 
     putdcc $idx "\[add\] Usage: .add <term> <expl>" 
  }
}

proc pxexpl:dcc:del {handle idx arg} {
  global explfile
  putcmdlog "#$handle# del $arg"
  if {$arg == ""} { 
    putdcc $idx "\[del\] Usage: .del <term>" 
  } elseif {[pxexpl:del [lindex $arg 0]]} { 
    putdcc $idx "\[del\] Explanation has been deleted." 
  } else { 
    putdcc $idx "\[del\] Error: no such term." 
  }
}

proc pxexpl:dcc:ls {handle idx arg} {
  global explfile maxterms cc
  putcmdlog "#$handle# ls $arg"
  set list [list]
  set total 0
  if {$arg == ""} {set arg "*"}
  set fd [open $explfile r]
  while {![eof $fd]} {
    set term [pxexpl:term [gets $fd]]
    if {$term != 0} {
      if {[string match [string tolower $arg] $term]} { 
        lappend list $term
      }
      incr total
    }
  }
  close $fd
  set matches [llength $list]
  if {$matches == 0} { 
    putdcc $idx "\[ls\] None of $total available terms matched your query." 
  } else { 
    putdcc $idx "\[ls\] \037$matches of $total terms matched your query:\037"
    set line [list]
    foreach term $list {
      if {[expr [string length $line] + [string length $term]] > 60} { 
        putdcc $idx "\[ls\] $line $term"
        set line [list]
      } else {
        set i [expr (11 * (([string length $term] / 12) + 1)) + ([string length $term] / 12)]
        while {[string length $term] < $i} { append term " " }
        append line " $term"
      }
    }
    if {[llength $line] != 0} { putdcc $idx "\[ls\] $line" }
  }
} 

proc pxexpl:get {term} {
  global explfile
  set fd [open $explfile r]
  while {![eof $fd]} {
    set input [gets $fd]
    if {[pxexpl:term $input] == [string tolower $term]} { 
      close $fd
      return "[string range $input [expr [string last ";" $input]+1] end] [string range $input [expr [string first ";" $input]+1] [expr [string last ";" $input]-1]]"
    }
  }
  close $fd
  return 0
}

proc pxexpl:add {term expl handle} {
  global explfile
  set added 0
  set ifd [open $explfile r]
  set ofd [open ${explfile}.tmp w]
  while {![eof $ifd]} {
    set fln [gets $ifd]
    set i [string compare [pxexpl:term $fln] [string tolower $term]]
    if {($i == 1) && (!$added)} {
      puts $ofd "[string tolower $term];$expl;$handle"
      set added 1
    }
    if {$i == 0} { 
       close $ifd
       close $ofd
       file delete ${explfile}.tmp
       return 0
    }
    puts $ofd $fln
  }
  if {!$added} { puts $ofd "[string tolower $term];$expl;$handle" }
  close $ifd
  close $ofd
  file rename -force ${explfile}.tmp $explfile  
  return 1
}

proc pxexpl:del {term} {
  global explfile
  set rval 0
  set term [string tolower $term]
  set ifd [open $explfile r]
  set ofd [open ${explfile}.tmp w]
  while {![eof $ifd]} {
    set fln [gets $ifd]
    if {[pxexpl:term $fln] == $term} { 
      set rval 1
    } else {
      puts $ofd $fln
    }
  }
  close $ifd
  close $ofd
  file rename -force ${explfile}.tmp $explfile
  return $rval
}

proc pxexpl:term {fstring} {
  set term [string range $fstring 0 [expr [string first ";" $fstring]-1]]
  if {$term != ""} { return $term } else { return 0 }
}

if {![file exists $explfile]} {
  set fd [open $explfile w]
  puts $fd "projectx;ProjectX is an organised group of scripters determined to help each other create, develop and refine scripts, like for example the Tcl script you see in action right now (ProjectX Explainer v$ver).  For more information about ProjectX, please point your browser to http://www.projectx.mx.dk/.;ProjectX"
  close $fd
}

putlog "\[ProjectX\] Explainer v$ver by Mentor <mentor@projectx.mx.dk> loaded."
